## \file
# \author 2012 Hsin-Yi Chen
# \brief A module for Command Process Logic
import sys
import argparse
import logging
from mazo import util

# \brief Interface of Command
class Command(object):

    ## \brief constructor
    def __init__(self):
        # init command args
        ## argparse.ArgumentParser
        self.argsparser = argparse.ArgumentParser()
        self._global_argkeys = []

        #@FIXME: mazo-launch can not handle verbose option
        self.add_argument('-v', '--verbose',
                          dest = 'verbose',
                          action='count',
                          isglobal=True)

        # init loggger
        logging.basicConfig(format='%(levelname)s:%(message)s')
        ## logging.Loger
        self.logger = logging.getLogger('command')

        self.set_argv()
        util.call_function(self, 'custom_arguments')

    ## \brief prepare cmd_args and maintask_args
    def prepare(self):
        ## command args
        self.cmd_args = self.parse_args()
        ## maintask args
        self.maintask_args = self.cmd_args
        self.handle_loglevel(self.cmd_args)

    ## \brief change log level by versbose option
    #  \param args argparse.Namespace command arguments
    def handle_loglevel(self, args):
        self.logger.setLevel(logging.ERROR)
        if args.verbose == 1:
            self.logger.setLevel(logging.INFO)
        elif args.verbose >= 2:
            self.logger.setLevel(logging.DEBUG)

    ## \brief set command argv
    #  \param argv arguments list (default:sys.argv)
    def set_argv(self, argv=None):
        if argv is not None:
            ## unparsed arguements list
            self.argv = argv
        else:
            self.argv = sys.argv[1:]

    ## \brief help of argparse.ArgumentParser.add_argument
    #  \param isglobal bool Extra param. True if it is a global argument
    #  \param *args
    #  \param *kwargs
    def add_argument(self, *args, **kwargs):
        try:
            isglobal = kwargs['isglobal']
        except KeyError:
            pass
        else:
            if isglobal:
                self._global_argkeys.append(kwargs['dest'])
            del(kwargs['isglobal'])
        return self.argsparser.add_argument(*args, **kwargs)

    ## \brief parse argv to args, argv can be set by set_argv function
    def parse_args(self):
        return self.argsparser.parse_args(self.argv)

    ## \brief transform args to dict
    #  \param args Namespace parsed args
    #  \param exclude list exclude keys
    #  \return dict
    def transform_args(self, args, exclude=[]):
        exclude = list(exclude) + self._global_argkeys
        #@NOTE: vars does no create a new object
        argvars = dict(vars(args))
        return util.filter_kwargs(argvars, exclude)
