## \file
# \brief Class composition tools.
#
# \author Thinker Li
# \url    http://www.codemud.net/~thinker/GinGin_CGI.py/show_id_doc/293
#
# This is a set of tools to composing classes.
# It provides tools to generate a new class from other classes.
# It also includes tools to re-link methods from composed classes
# for the new class.
#
# With class composition tools, programmer can seperate concerns
# into classes.  The classes are tested independently and composed
# to create new classes from classes with different aspect.
#
# A composed class is dealing about an aspect of concern.  It
# is template class that some methods may are not implemented.
# Programmers use classes to generate a new class should
# re-link not implemented methods to the method provides by other
# composed classes to complete the implementation of the composed
# classes.


## \brief Composite classes to create a new class.
#
# It makes composed classes as base classes of the new class.
def composite(clz_name, *clazzs):
    new_clz = type(clz_name, clazzs, {})
    return new_clz

## \brief Re-link a source method to destination method.
#
# It relinks methods of a composition class.
def link(composition, src_method, dst_method):
    setattr(composition, src_method.__name__, dst_method)

## \brief Relink method pairs of a composition class.
def slink(composition, src_dst_pairs):
    for src, dst in src_dst_pairs:
        setattr(composition, src.__name__, dst)
