# \author 2012 Hsin-Yi Chen
from mazo.role import worker

## \brief metaclass of Extension
#
# To create composed class of Extension
#
# used attributes:
#
#   - responsible_for: required Job interface classes
#   - interface:    optional user interface classes
class ExtensionType(type):

    ## \brief create a class that inhereince classed defined in
    #         responsible_for and interface attribute of dct.
    def __new__(mbs, name, bases, dct):
        try:
            required_jobs = dct['responsible_for']
            if type(required_jobs) is not tuple and \
               type(required_jobs) is not list:
                required_jobs = [required_jobs]
        except KeyError:
            raise Exception('ERR')

        try:
            interface = dct['interface']
        except KeyError:
            interface = None
        if interface:
            name = '{}_{}'.format(interface.__name__, name)
            cls = worker(name, required_jobs, [interface])
        else:
            cls = worker(name, required_jobs)
        # re-link attributes, methods defined in extension
        for key in dct:
            # only rebind public interface/attributes
            setattr(cls, key, dct[key])
        return cls
