import sys
import imp
import os
import argparse

from mazo.extensionlib import ExtensionType
from mazo.missionlib import DoJob, FormatMetaDataJob
from mazo.cmdlib import Command

## \brief Invoker of Launch
class ExtensionLaunch(object):
    __metaclass__ = ExtensionType
    responsible_for = DoJob
    interface = Command

    def custom_arguments(self):
        self.add_argument('suite', help='extension suite name')
        self.add_argument('extension', help='extension name')
        self.add_argument('extension_args', nargs=argparse.REMAINDER, help='extension args')

    def maintask(self, args):
        # import extension module
        try:
            if os.path.isdir(args.suite):
                mod = self.import_src(args)
            else:
                mod = self.import_mod(args)
        except ImportError, e:
            self.logger.error('can not find suite {0}'.format(args.suite))
            self.logger.error(e)
            exit()
        # import extension class
        clsname = 'Extension'+args.extension.capitalize()
        try:
            compos_cls = getattr(mod, clsname)
        except AttributeError:
            self.logger.error('{0} does not have extension {1}'.format(
                        args.suite,
                        args.extension))
            self.logger.debug('Failed to load class {0} in {1}'.format(clsname, mod))
            exit()

        # run extension
        ext=compos_cls()
        # make sure we are launching any extension but me
        isme = compos_cls == self.__class__
        if isme:
            print 'Hey! It is me!'
        else:
            ext.set_argv(args.extension_args)
            ext.do()

    def import_mod(self, args):
        return __import__('mazo.extensions.'+args.suite,
                             fromlist=['mazo.extensions'])

    def import_src(self, args):
        mod_path = os.path.join(args.suite, '__init__.py')
        self.logger.debug('Found extension module in path {0}'.format(mod_path))
        return imp.load_source(os.path.basename(args.suite), mod_path)

class ExtensionFormatmetadata(object):
    __metaclass__ = ExtensionType
    responsible_for = DoJob, FormatMetaDataJob
    interface = Command

    def maintask(self, args):
        pass

    def custom_arguments(self):
        self.add_argument('target')
        self.add_argument('--title')
        self.add_argument('--metadata', nargs=2, metavar=('name','path'), action='append', required=True)
        self.add_argument('--template', nargs=2, metavar=('name','path'), action='append')

## \brief execute extension that interface is Command
#
# \param suite extension suite name
# \param ext extension name
def runcmd(suite, ext=None):
    launcher = ExtensionLaunch()
    # use file name as program name
    launcher.argsparser.prog = sys.argv[0]
    # append extension args
    launcher.set_argv([suite, ext] + sys.argv[1:])
    launcher.do()
