# \file misc usefule funtions
# \author 2012 Hsin-Yi Chen
import datetime
import ConfigParser
import os

## \breif helper to get object attributes
#  \param obj PyObj any type of object
#  \param query str attribute name  ex. 'bug.tags'
#  \return the value of PyObj attribute given by query
def get_obj_attrval(obj, query):
    if query.startswith('.'):
        raise TypeError('bad query name {0}'.format(query))
    if '.' not in query:
        return getattr(obj, query)
    else:
        attrnames = query.split('.')
        attrval = getattr(obj, attrnames[0])
        for attrname in attrnames[1:]:
            attrval = getattr(attrval, attrname)
        return attrval

## \brief call a function of an object if function exists
#  \param obj PyObj any type of object
#  \param name str  method name
#  \param *args function args
#  \param *kwargs function keyword args
def call_function(obj, name, *args, **kwargs):
    try:
        func = getattr(obj, name)
    except AttributeError:
        pass
    else:
        if callable(func):
            func(*args, **kwargs)

## \brief get launchpad login id
def getbzrlogin():
    path = os.path.expanduser('~/.bazaar/bazaar.conf')
    if not os.path.exists(path):
        return None
    cfg = ConfigParser.ConfigParser()
    cfg.read(os.path.expanduser('~/.bazaar/bazaar.conf'))
    try:
        return cfg.get('DEFAULT', 'launchpad_username')
    except ConfigParser.NoOptionError:
        return None

## \brief get a dict without special elements.
#  \param original dict original dict
#  \param exclude tuple the keys of element will be removed
#  \return dict
def filter_kwargs(original, exclude):
    if not exclude:
        raise TypeError('the exclude keys is not sepcified')
    (orig, new)  = split_kwargs(original, exclude)
    return orig

## \brief split dict to 2 dict
#  \param original dict original dict
#  \param antohers tuple the keys of element will be removed
#  \return tuple (original, new)
def split_kwargs(original, anothers):
    new = {}
    for key in anothers:
        try:
            new[key] = original.pop(key)
        except KeyError:
            pass
    return (original, new)

## \brief get current week number
def get_week_num():
    today_calendar = datetime.datetime.now().isocalendar()
    week_num = today_calendar[1]
    week_day = today_calendar[2]
    #tim: if we're at sunday,monday,tuesday,wednesday provide last week's week number,
    #     if we're at thursday,friday,saturday we provide this week's week num.
    if week_day < 4:
        week_num -= 1
    return week_num
