#!/usr/bin/env python
# -*- encoding=utf8 -*-
#
# Author 2011 Hsin-Yi Chen
import os
import tempfile
import unittest

from mazo.extensions.lp_cli import Conditions

class ConditionsTestCase(unittest.TestCase):

    def setUp(self):
        self.filename = os.path.join(tempfile.gettempdir(), 'conditions.yml')

    def tearDown(self):
        os.remove(self.filename)

    def _read(self):
        return Conditions.load(self.filename)

    def _write(self, lines):
        with open(self.filename, 'w') as fileobj:
            fileobj.write('\n'.join(lines + ['']))

    def test_get(self):
        self._write([
            'conditions:',
            '    next_must:',
            '        status:',
            '            - New',
            '            - Confirmed',
            '        modified_since: $date_before_now'
        ])
        conds = self._read()
        self.assertEquals(None, conds.get('next'))
        cond = conds.get('next_must')
        self.assertEquals(['New', 'Confirmed'], cond.get('status'))
        self.assertEquals(conds.shared_info['date_before_now'], cond.get('modified_since'))

def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(ConditionsTestCase, 'test'))
    return suite

if __name__ == '__main__':
    unittest.main(defaultTest='suite')
