#!/usr/bin/env python
# -*- encoding=utf8 -*-
#
# Author 2012 Hsin-Yi Chen
import os
import mock
import unittest

from mazo import util

class get_obj_attrvalTestCase(unittest.TestCase):

    def setUp(self):
        self.mock = mock.Mock()
        self.mock.title = 'fake bug'
        self.mock.bug.tags = range(1,5)

    ## \brief testing get the value of object attribute.
    def test_main(self):
        self.assertEquals(range(1,5), util.get_obj_attrval(self.mock, 'bug.tags'))
        self.assertEquals('fake bug', util.get_obj_attrval(self.mock, 'title'))
        # check side effect

    ## \brief test the side effect handling of get the value of object attribute
    def test_side_effect(self):
        _list = []
        self.assertRaises(AttributeError, util.get_obj_attrval, _list, 'nonexists')
        self.assertRaises(TypeError, util.get_obj_attrval,self.mock, '.bug.tags')

class filter_kwargsTestCase(unittest.TestCase):

    def test_main_with_exclude(self):
        excepted = {'a':1}
        input_data = {'a':1,
                      'b':2,
                      'c':3}
        exclude = ('b', 'c')
        self.assertEquals(excepted, util.filter_kwargs(input_data, exclude))

    def test_main_without_exclude(self):
        input_data = {'a':1,
                      'b':2,
                      'c':3}
        excepted = input_data
        exclude = ()
        self.assertRaises(TypeError, util.filter_kwargs,input_data, exclude)


def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(get_obj_attrvalTestCase, 'test'))
    suite.addTest(unittest.makeSuite(filter_kwargsTestCase, 'test'))
    return suite

if __name__ == '__main__':
    unittest.main(defaultTest='suite')
